# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.3 $

package Controller;

sub NAME () { 'Controller'; }

# Encode States (all encoders)

sub Running           { 'Controller__Running'; }
sub Halted            { 'Controller__Halted'; }

# Encode Messages

sub EncodeState       { 'Controller__EncodeState'; }
sub EncodeHalt        { 'Controller__EncodeHalt'; }
sub EncodeResume      { 'Controller__EncodeResume'; }

# Encode Responses

sub EncodeStatus      { 'Controller__EncodeStatus'; }
sub EncodeDiagnostic  { 'Controller__EncodeDiagnostic'; }

# Encoder States (each individual encoder)

sub Encoding          { 'Controller__Encoding'; }
sub Dead              { 'Controller__Dead'; }
sub Idle              { 'Controller__Idle'; }
#sub Halted            { 'Controller__Halted'; }

# EncodeStatus message are sent when there is a change in the state of
# the encoding engine. Message `parms' are a hash ref of `state',
# `encoders' and `tracks'. Where `state' is the overall encoding
# engine state. `encoders' is an array ref whose elements are a hash
# ref describing each encoder. Containing `state' as above, a `handle'
# to the encoder plus optional members by state as outlined
# below. `tracks' is an array ref whose elements are hash refs
# containing the `track' that requires encoding, the `rate' at which
# it will be encoded and optional `handle', `start' time and
# `percentageComplete' if it is in the process of being encoded.

#   Encoding - This encoder is working on an encode at present.

#   Idle - This encoder would like to be encoding but for some
#   `reason' (which is a small description (<40chars) suitable for
#   user display) it can't.

#   Dead - The encoder cannot be contacted.

#   Halted - The encoder has stopped working by user request.

# EncodeDiagnostic messages are sent to describe 'out of band' events
# that occur when encoding, typically an error with the encoder or
# lack of space. Message `parms' are a hash ref of `handle', `summary'
# and `diagnostic'.

#   `handle' - Identifies the thing that had the problem.

#   `summary' - Short description of the problem.

#   `diagnostic' - Complete information.


# Rip Messages

sub RipState          { 'Controller__RipState'; }

# Rip Responses

sub RipStatus         { 'Controller__RipStatus'; }
sub RipDiagnostic     { 'Controller__RipDiagnostic'; }

# Ripper Messages

sub RipperRip         { 'Controller__RipperRip'; }
sub RipperAdd         { 'Controller__RipperAdd'; }
sub RipperRemove      { 'Controller__RipperRemove'; }

# Ripper States (each individual encoder)

sub Ripping           { 'Controller__Ripping'; }
#sub Dead              { 'Controller__Dead'; }
#sub Idle              { 'Controller__Idle'; }
#sub Halted            { 'Controller__Halted'; }

# RipStatus message are sent when there is a change in the state of
# the ripping engine. Message `parms' are an array ref of ripper
# descriptions. Where each description is a hash ref containing
# `state' as above, a `handle' to the ripper plus optional members by
# state as outlined below. One optional member is `tracks' which will
# be present when there is a CD in the drive and will be a hash ref
# indexed by track index containing the members, `index', `want',
# `ripped' and `encoded'. `want' signifies that the user wants to rip
# this track. `ripped' denotes that this track has been
# ripped. `encoded' marks it as fully encoded for all rates.

#   Ripping - This ripper is working on an rip at present.

#   Idle - This ripper would like to be ripping but for some
#   `reason' (which is a small description (<40chars) suitable for
#   user display) it can't.

#   Dead - The ripper cannot be contacted.

#   Halted - The ripper has stopped working by user request.

# RipDiagnostic messages are similiar to EncodeDiagnostic messages.

1;
