# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.4 $

package GtkEx::BusyIcon;

use Gtk;
@ISA = qw( Gtk::EventBox );
use Trace;
use strict;

my $_pixmaps = 1;

sub new {
    my $proto = shift;
    my $type = ref($proto) || $proto;
    my %parms = @_;

    my $widget = $type->SUPER::new();
    if ($widget) {
	if (ref($widget) ne $proto) {
	    bless $widget, $type;
	}
	my $box = Gtk::HBox->new(0, 0);
	$box->border_width(0);
	$box->show();
	$widget->add($box);

	$widget->{'rotates'} = [];
	if ($_pixmaps) {
	    my $window = $parms{'window'} || die;
	    my $style = $parms{'style'} || die;
	    my $pixmaps = $parms{'pixmaps'} || die;
	    foreach my $xpm (@{ $pixmaps }) {
		my ($pixmap, $mask) = Gtk::Gdk::Pixmap->create_from_xpm($window->window, $style->bg('normal'), $xpm);
		my $p = Gtk::Pixmap->new($pixmap, $mask);
		$p->hide();
		$box->pack_start($p, 0, 0, 0);
		push @{ $widget->{'rotates'} }, $p;
	    }
	    $widget->{'rotates'}->[0]->show;
	}
	else {
	    foreach my $ch ('|', '/', '-', '\\') {
		my $l = Gtk::Label->new($ch);
		$l->set_usize(20, 0);
		$l->hide();	    
		$box->pack_start($l, 0, 0, 0);
		push @{ $widget->{'rotates'} }, $l;
	    }
	    my $l = Gtk::Label->new('');
	    $l->set_usize(22, 22);
	    $l->show();	    
	    $box->pack_start($l, 0, 0, 0);
	    $widget->{'inactive'} = $l;
	}
	$widget->{'state'} = 'stopped';
	$widget->{'current'} = 0;
	$widget->{'id'} = 1;
	$widget->{'tooltips'} = Gtk::Tooltips->new;
	$widget->{'tooltips'}->enable;
    }
    return $widget;
}

sub start {
    my $widget = shift;
    my %parms = @_;

    $widget->{'id'}++;
    $widget->{'stack'}->{ $widget->{'id'} } = { id => $widget->{'id'}, msg => $parms{'msg'}, 
						timeout => defined($parms{'timeout'}) ? time + $parms{'timeout'} : undef, 
						callback => $parms{'callback'}, parms => \%parms };
    $widget->_updateMsg;

    if ($widget->{'state'} eq 'stopped') {
	$widget->{'state'} = 'started';
	$widget->{'inactive'}->hide if (!$_pixmaps);
	if ($widget->step) {
	    Gtk->timeout_add(50, sub { $_[0]->step }, $widget);
	}
    }

    return $widget->{'id'};
}

sub stop {
    my $widget = shift;
    my %parms = @_;
    my $id = $parms{'id'} || die;
    delete $widget->{'stack'}->{ $id };
    $widget->_updateMsg;
}

sub changeMsg {
    my $widget = shift;
    my %parms = @_;
    my $id = $parms{'id'} || die;
    my $msg = $parms{'msg'} || die;
    $widget->{'stack'}->{ $id }->{'msg'} = $msg;
}

sub step {
    my $widget = shift;
			 
    my $now = time;
    foreach my $busy (values(%{ $widget->{'stack'} })) {
	if (defined($busy->{'timeout'})) {
	    if ($now > $busy->{'timeout'}) {
		Trace::debug('Timeout for %d', $busy->{'id'}) if Trace::check(Trace::DEBUG4);
		if ($busy->{'callback'}) {
		    &{ $busy->{'callback'} }(id => $busy->{'id'}, @{ $busy->{'parms'} });
		}
		delete $widget->{'stack'}->{ $busy->{'id'} };
		$widget->_updateMsg;
	    }
	}
    }
    if (! scalar keys %{ $widget->{'stack'} }) {
	$widget->{'state'} = 'stopped';
	$widget->{'rotates'}->[ $widget->{'current'} ]->hide if (!$_pixmaps);
	$widget->{'inactive'}->show if (!$_pixmaps);
	$widget->{'tooltips'}->set_tip($widget, '', '');
	return 0;
    }

    $widget->{'rotates'}->[ $widget->{'current'} ]->hide;
    $widget->{'current'}++; 
    $widget->{'current'} = 0 if ($widget->{'current'} > $#{ $widget->{'rotates'} }); 
    $widget->{'rotates'}->[ $widget->{'current'} ]->show;
    return 1;
};

sub _updateMsg {
    my $widget = shift;
    $widget->{'tooltips'}->set_tip($widget, join(', ', grep { defined($_) } map { $_->{'msg'} } values(%{ $widget->{'stack'} })), '');
}

1;


