# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.1 $

package GtkEx::EntryViaFileSelection;

# Two column edit table

use Gtk;
@ISA = qw( Gtk::Entry );
use Trace;
use strict;

sub new { 
    my $proto = shift;
    my $type = ref($proto) || $proto;

    my $entry;
    $entry = $type->SUPER::new($_[0]);

    return undef if (!defined($entry));

    my $hbox = Gtk::HBox->new(0, 2);
    $hbox->border_width(0);
    $hbox->show();
    $hbox->pack_start($entry, 1, 1, 0);

    my $button = Gtk::Button->new('Choose...');
    $button->signal_connect('clicked', $proto.'::_choose', $entry);
    $button->show();
    $hbox->pack_start($button, 0, 0, 0);

    if (ref($entry) ne $proto) {
	bless $entry, $type;
    }

    $entry->{'_button'} = $button;
    $entry->{'_widget'} = $hbox;
    $entry->{'_windowTitle'} = $_[1];
    
    return $entry;
}

sub container {
    $_[0]->{'_widget'};
}

sub _choose {
    my $widget = shift;
    my $entry = shift;
    
    $entry->{'_button'}->set_sensitive(0);

    my $fs = Gtk::FileSelection->new($entry->{'_windowTitle'});
	
    $fs->signal_connect('destroy', sub { $entry->{'_button'}->set_sensitive(1); });
    $fs->signal_connect('delete_event', sub { $entry->{'_button'}->set_sensitive(1); });

    $fs->ok_button->signal_connect('clicked', ref($entry).'::_set', $entry, $fs);
    $fs->ok_button->signal_connect('clicked', ref($entry).'::_destroy', $entry, $fs);
    $fs->cancel_button->signal_connect('clicked', ref($entry).'::_destroy', $entry, $fs);
    $fs->show;
    return 1;
}

sub _set {
    my $widget = shift;
    my $entry = shift;
    my $fs = shift;

    $entry->set_text($fs->get_filename);
    return 1;
}

sub _destroy {
    my $widget = shift;
    my $entry = shift;
    my $fs = shift;

    $entry->{'_button'}->set_sensitive(1);
    $fs->destroy;
    return 1;
}

1;
