# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.4 $

package GtkEx::TableEdit;

# Two column edit table

use Gtk;
@ISA = qw( Gtk::Table );
use Trace;
use strict;

sub new { 
    my $proto = shift;
    my $type = ref($proto) || $proto;

    my %parms = @_;
    my $table;
    $table = $type->SUPER::new(0, 2, 0);

    return undef if (!defined($table));

    # set_row_spacing() etc calls do not work, hack with padding
    if ($parms{'rowSpacing'}) {
	$table->{'_rowSpacing'} = $parms{'rowSpacing'};
    }
    else {
	$table->{'_rowSpacing'} = 0;
    }
    if ($parms{'colSpacing'}) {
	$table->{'_colSpacing'} = $parms{'colSpacing'};
    }
    else {
	$table->{'_colSpacing'} = 0;
    }
    if (ref($table) ne $proto) {
	bless $table, $type;
    }
    if ($parms{'tooltips'}) {
	$table->{'_tooltips'} = $parms{'tooltips'};
    }

    $table->{'_fields'} = [];
    $table->{'_rows'} = 0;

    return $table;
}

sub appendRow {
    my $table = shift;
    my %parms = @_;

    my $pos = $table->{'_rows'}++;
    $table->resize(1, 0);

    # Justify the label in the upper right hand corner

    my $box1 = Gtk::VBox->new(0, 0);
    $box1->border_width(0);
    $box1->show();

    my $box2 = Gtk::HBox->new(0, 0);
    $box2->border_width(0);
    $box2->show();
    $box1->pack_start($box2, 0, 1, 0);

    my $box3 = Gtk::HBox->new(0, 0);
    $box3->border_width(0);
    $box3->show();
    $box1->pack_start($box3, 1, 1, 0);

    my $box4 = Gtk::HBox->new(0, 0);
    $box4->border_width(0);
    $box4->show();
    $box2->pack_start($box4, 1, 1, 0);

    my $label = Gtk::Label->new($parms{'title'});
    $label->set_justify('left');
    $label->set_line_wrap(1);
    $label->show();

    my $event = Gtk::EventBox->new;
    $event->show;
    $event->add($label);

    $box2->pack_start($event, 0, 0, 0);

    if (defined($parms{'tip'}) && defined($table->{'_tooltips'})) {
	$table->{'_tooltips'}->set_tip($event, $parms{'tip'}, '');
    }

    my $box5 = Gtk::HBox->new(0, 0);
    $box5->border_width(0);
    $box5->show();
    $box5->pack_start($parms{'widget'}, 0, 0, 0);

    $table->{'_fields'}->[ $pos ] = { label => $label, widget => $parms{'widget'} };

    # XPad the second column only gives the spacing required with a
    # harmless bit on the right.

    # YPad odd rows gives the row spacing required.

    $table->attach($box1, 0, 1, $pos, $pos + 1, 'fill', 'fill', $table->{'_colSpacing'} / 2, 
		   ($pos % 2) ? $table->{'_rowSpacing'} : 0);
    $table->attach($box5, 1, 2, $pos, $pos + 1, ['expand','fill'], 'fill', $table->{'_colSpacing'} / 2,  
		   ($pos % 2) ? $table->{'_rowSpacing'} : 0);

}

# placeRow() - Overwrite an existing table entry at 'row'.

sub placeRow {
    my $table = shift;
    my %parms = @_;

    if (defined($table->{'_fields'}->[ $parms{'row'} ])) {
	my $field = $table->{'_fields'}->[ $parms{'row'} ];
	my $labelC = $field->{'label'}->parent;
	$labelC->remove($field->{'label'});
	my $widgetC = $field->{'widget'}->parent;
	$widgetC->remove($field->{'widget'});
	my $label = Gtk::Label->new($parms{'title'});
	$label->set_justify('left');
	$label->set_line_wrap(1);
	$label->show();
	$labelC->add($label);
	$widgetC->pack_start($parms{'widget'}, 0, 0, 0);
	$table->{'_fields'}->[ $parms{'row'} ]->{'label'} = $label;
	$table->{'_fields'}->[ $parms{'row'} ]->{'widget'} = $parms{'widget'};
    }
}

sub activateRow {
    my $table = shift;
    my %parms = @_;

    $table->{'_fields'}->[$parms{'row'}]->{'label'}->set_sensitive(1);
    $table->{'_fields'}->[$parms{'row'}]->{'widget'}->set_sensitive(1);
}

sub deactivateRow {
    my $table = shift;
    my %parms = @_;

    $table->{'_fields'}->[$parms{'row'}]->{'label'}->set_sensitive(0);
    $table->{'_fields'}->[$parms{'row'}]->{'widget'}->set_sensitive(0);
}

1;
