# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.4 $

package GtkEx::Utility;

use Gtk;
use Trace;
use strict;

sub ackPopup {
    my %parms = @_;
    my $message = $parms{'msg'} || die;
    my $func = $parms{'func'} || sub { 1; };

    if ($parms{'truncate'} && (length $message > $parms{'truncate'})) {
	$message = (substr $message, 0, $parms{'truncate'}).'...';
    }

    my $dialog = Gtk::Dialog->new;
    $dialog->set_title($message);

    $dialog->signal_connect('delete_event', sub { &{ $func } });

    my $label = Gtk::Label->new($message);
    $label->set_justify('left');
    $label->set_line_wrap(1);
    $label->show();

    my $okButton = Gtk::Button->new('OK');
    $okButton->signal_connect('clicked', sub { &{ $func }; $dialog->destroy; });
    $okButton->show();

    $dialog->vbox->border_width(10);
    $dialog->vbox->add($label);
    $dialog->action_area->pack_start($okButton, 0, 0, 0);
    $dialog->show;
    return 1;
}

# choicePopup - Display a choice window with a `msg', can give a
# `title', different button `labels' other than Ok and Cancel and user
# called functions, `okFunc', `cancelFunc'. Other `buttons' can be
# specified with a `label' and `func'.

sub choicePopup {
    my %parms = @_;
    my $message;
    my $title;
    my $widget;

    if (defined($parms{'msg'})) {
	$message = $parms{'msg'};
    }
    if (defined($parms{'widget'})) {
	$widget = $parms{'widget'};
    }
    $title = defined($parms{'title'}) ? $parms{'title'} : $message;

    my @labels = defined($parms{'labels'}) ? @{ $parms{'labels'} } : ('Ok', 'Cancel');

    my $choice = Gtk::Window->new('dialog');
    $choice->set_title($title);
    $choice->signal_connect('delete_event', defined($parms{'cancelFunc'}) ? sub { &{ $parms{'cancelFunc'} } } : sub { 1; });
    my $vbox = Gtk::VBox->new(0, 0);
    $vbox->border_width(5);
    $vbox->show();
    $choice->add($vbox);

    # Message

    if (defined($message)) {
	my $label = Gtk::Label->new($message);
	$label->set_justify('left');
	$label->set_line_wrap(1);
	$label->show();
	$vbox->pack_start($label, 1, 1, 0);

	my $hsep = Gtk::HSeparator->new;
	$hsep->show;
	$vbox->pack_start($hsep, 0, 0, 5);
    }

    if (defined($widget)) {

	$vbox->pack_start($widget, 1, 1, 0);

	my $hsep = Gtk::HSeparator->new;
	$hsep->show;
	$vbox->pack_start($hsep, 0, 0, 5);
    }

    # Buttons

    my $hbox = Gtk::HButtonBox->new();
    $hbox->set_layout_default('end');
    $hbox->set_spacing(5);
    $hbox->set_child_ipadding(15, 0);
    $hbox->set_child_size(0, 0);
    $hbox->show();
    $vbox->pack_start($hbox, 0, 0, 0);

    if (defined($parms{'buttons'})) {
	foreach my $button (@{ $parms{'buttons'} }) {
	    my $b = Gtk::Button->new($button->{'label'});
	    $b->signal_connect('clicked', $button->{'func'});
	    $b->show();
	    $hbox->pack_start($b, 0, 0, 0);
	}
    }

    my $okButton = Gtk::Button->new($labels[0]);
    my $okFuncR = defined($parms{'okFunc'}) ? sub { if (&{ $parms{'okFunc'} }) { $choice->destroy; } } : sub { $choice->destroy; };
    $okButton->signal_connect('clicked', $okFuncR);
    $okButton->show();
    $hbox->pack_start($okButton, 0, 0, 0);

    my $cancelButton = Gtk::Button->new($labels[1]);
    my $cancelFuncR = defined($parms{'cancelFunc'}) ? sub { if (&{ $parms{'cancelFunc'} }) { $choice->destroy; } } : sub { $choice->destroy; };
    $cancelButton->signal_connect('clicked', $cancelFuncR);
    $cancelButton->show();
    $hbox->pack_start($cancelButton, 0, 0, 0);

    $choice->show;
    return $choice;
}

# listPadding - Setting a lists column widths using the available
# space always results in an overly wide list (scrollbar). Trial and
# error gives us this good padding number based on the number of
# columns.

sub listPadding {
    my %parms = @_;

    return ($parms{'columns'} + 1) * 8;
}

1;
