# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.3 $

package Gui::Page;

# Pages that can be attached to GUI windows.

use strict;
use Trace;

sub new {
    my $proto = shift;
    my $type = ref($proto) || $proto;
    my %parms = @_;
    my $self = {};
    my $page = $self->{'_page'} = {};

    $page->{'title'} = $parms{'title'} || die;
    $page->{'widget'} = $parms{'widget'} || die;
    $page->{'gui'} = $parms{'gui'} || die;

    bless $self, $type;

    $page->{'widget'}->set_user_data($self);
    $page->{'hidden'} = 1;
    $page->{'widget'}->ref;

    return $self;
}

sub attach {
    my $self = shift;
    my $page = $self->{'_page'};
    my %parms = @_;

    $page->{'window'} = $parms{'window'} || die;
    $self->show;
    return 1;
}

sub deattach {
    my $self = shift;
    my $page = $self->{'_page'};

    delete $page->{'window'};
    $self->hide;
    return 1;
}

sub hide {
    my $self = shift;
    my $page = $self->{'_page'};
    $page->{'widget'}->ref;
    $page->{'hidden'} = 1;
    return 1;
}

sub show {
    my $self = shift;
    my $page = $self->{'_page'};
    $page->{'widget'}->unref;
    $page->{'hidden'} = 0;
    return 1;
}

sub destroy {
    my $self = shift;
    my $page = $self->{'_page'};
    $page->{'widget'}->destroy;
    return 1;
}

sub hidden    { $_[0]->{'_page'}->{'hidden'}; }
sub widget    { $_[0]->{'_page'}->{'widget'}; }
sub window    { $_[0]->{'_page'}->{'window'}; }
sub title     { $_[0]->{'_page'}->{'title'}; }
sub gui       { $_[0]->{'_page'}->{'gui'}; }
sub default_width         { 600; }
sub default_height        { 600; }

# aspect - Window has been moved or resized

sub aspect {
    my $page = shift;
}

# event - Debug signals/events

sub event {
    my $self = shift;
    my $page = $self->{'_page'};
    my $widget = shift;
    my $event = shift;

    Trace::debug(ref($widget).'::'.$event->{'type'}) if Trace::check(Trace::DEBUG3);
    if ($ENV{GUI_EVENT_DEBUG} eq 'v') {
      require 'dumpvar.pl';
      main::dumpValue($event);
      main::dumpValue([$event->{'window'}->get_size]) if ($event->{'window'});
      $| = 1;
    }

    return 0;
}

1;
