# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.1 $

package Gui::Page::Diagnostic;

use Gui::Page;
@ISA = qw( Gui::Page );
use strict;
use ControllerH;
use Gtk;
use Trace;

sub new {
    my $proto = shift;
    my $type = ref($proto) || $proto;
    my %parms = @_;

    my $vbox = Gtk::VBox->new(0, 0);
    $vbox->border_width(0);
    $vbox->show();

    my $page = $type->SUPER::new(@_, title => 'Diagnostics', widget => $vbox);

    my $scroll = Gtk::ScrolledWindow->new(undef, undef);
    $scroll->set_policy('automatic', 'automatic');
    $scroll->show();

    my $text = Gtk::Text->new(undef, undef);
    $text->show;
    $page->{'text'} = $text;

    $scroll->add($text);
    $vbox->pack_start($scroll, 1, 1, 0);

    my $ripDiagnosticH = sub {
	shift;
	my %tParms = @_;
	my $parms = $tParms{'parms'};
	$page->add(type => 'Ripper', name => $parms->{'handle'}->key, summary => $parms->{'summary'}, diagnostic => $parms->{'diagnostic'});
	return;	
    };
    $page->gui->register(name => Controller::RipDiagnostic, handler => $ripDiagnosticH);

    my $encodeDiagnosticH = sub {
	shift;
	my %tParms = @_;
	my $parms = $tParms{'parms'};
	$page->add(type => 'Encoder', name => $parms->{'handle'}->key, summary => $parms->{'summary'}, diagnostic => $parms->{'diagnostic'});
	return;	
    };
    $page->gui->register(name => Controller::EncodeDiagnostic, handler => $encodeDiagnosticH);

    return $page;
}

sub default_width         { 600; }
sub default_height        { 400; }

sub add {
    my $page = shift;
    my %parms = @_;

    my $msg = sprintf("%s: %s (%s): %s\n", POSIX::strftime('%d/%m %H:%M:%S', localtime(time)), 
		      $parms{'type'}, $parms{'name'}, $parms{'summary'});
    if (defined($parms{'diagnostic'})) {
	$msg .= $parms{'diagnostic'}."\n";
    }
    $msg .= "\n";

    my $text = $page->{'text'};
    $text->freeze;
    $text->set_point(0);
    $text->insert(undef, $text->style->black, undef, $msg);

    # Cut off at 8000 chars

    my $length = $text->get_length;
    if ($length > 8000) {
	$text->set_point($length);
	$text->backward_delete($length - 8000);
    }
    $text->thaw;

    if ($page->hidden) {
	$page->gui->createWindow($page);
    }

    return 1;
}

1;
