# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.1 $

package Gui::Page::Encoding;

use Gui::Page;
@ISA = qw( Gui::Page );
use ControllerH;
use strict;
use Gtk;
use Trace;

sub new {
    my $proto = shift;
    my $type = ref($proto) || $proto;
    my %parms = @_;

    my $vbox = Gtk::VBox->new(0, 0);
    $vbox->border_width(0);
    $vbox->show();

    my $page = $type->SUPER::new(@_, title => 'Encoding Status', widget => $vbox);

    my $encodeStatusH = sub { 
	shift; 
	my %tParms = @_;
	my $parms = $tParms{'parms'};
	$page->encodeStatus($parms); 
    };
    $page->gui->register(name => Controller::EncodeStatus, handler => $encodeStatusH);

    # Info

    my $infoLabel = $page->{'infoLabel'} = Gtk::Label->new('');
    $infoLabel->set_justify('left');
    $infoLabel->hide();
    $vbox->pack_start($infoLabel, 0, 0, 0);

    # Pane

    my $pane = $page->{'pane'} = Gtk::VPaned->new();
    $pane->signal_connect('event', sub { $page->event(@_); }) if ($page->gui->eventDebug);
    $pane->set_gutter_size(4);
    $pane->set_handle_size(4);
    $pane->border_width(0);
    $pane->show();
    $vbox->pack_start($pane, 1, 1, 0);

    # Tracks

    my $trackScroll = Gtk::ScrolledWindow->new(undef, undef);
    $trackScroll->set_policy('automatic', 'automatic');
    $trackScroll->show();
    my $trackCList = $page->{'trackCList'} = Gtk::CList->new_with_titles(' ', 'Track', 'Artist', 'Album', 'Rate');
    $trackCList->set_selection_mode('browse');
    $trackCList->set_row_height($page->gui->rowHeight);
    $trackScroll->add($trackCList);
    $trackCList->show();
    $trackCList->column_titles_passive();
    $pane->pack1($trackScroll, 1, 1);

    # Encoders

    my $encoderScroll = Gtk::ScrolledWindow->new(undef, undef);
    $encoderScroll->set_policy('automatic', 'automatic');
    $encoderScroll->show();
    my $encoderCList = $page->{'encoderCList'} = Gtk::CList->new_with_titles('Encoder', ' ');
    $encoderCList->set_selection_mode('browse');
    $encoderCList->set_row_height($page->gui->rowHeight);
    $encoderScroll->add($encoderCList);
    $encoderCList->show();
    $encoderCList->column_titles_passive();
    $pane->pack2($encoderScroll, 0, 0);

    # Buttons

    my $buttonBox = Gtk::HBox->new(0, $page->gui->spacing);    
    $buttonBox->show();
    $vbox->pack_start($buttonBox, 0, 0, 0);

    # halt / resume

    $page->{'haltButton'} = my $haltButton = Gtk::Button->new();
    $haltButton->signal_connect('clicked', sub { $page->haltOrResume(@_); });
    $haltButton->show();
    $page->{'haltLabel'} = my $haltLabel = Gtk::Label->new('');
    $haltLabel->set_text('Halt');
    $haltLabel->set_justify('left');
    $haltLabel->show();
    $haltButton->add($haltLabel);
    $buttonBox->pack_start($haltButton, 1, 1, 0);

    $page->{'state'} = Controller::Running;
    $page->{'previouslyAttached'} = 0;
    return $page;
}

sub attach {
    my $page = shift;

    $page->SUPER::attach(@_);

    if (!$page->{'previouslyAttached'}) {
	my $available = $page->window->width;
	$available -= GtkEx::Utility::listPadding(columns => 5);
	my $list = $page->{'trackCList'};
	$list->set_column_width(0, int($available * 0.12));
	$list->set_column_width(1, int($available * 0.26));
	$list->set_column_width(2, int($available * 0.26));
	$list->set_column_width(3, int($available * 0.26));
	$list->set_column_width(4, int($available * 0.10));

	$available = $page->window->width;
	$available -= GtkEx::Utility::listPadding(columns => 2);;
	$list = $page->{'encoderCList'};
	$list->set_column_width(0, int($available * 0.30));
	$list->set_column_width(1, int($available * 0.70));
	
	# Lower pane gets 20%
	$page->{'pane'}->set_position($page->window->height * 0.80);
	$page->{'previouslyAttached'} = 1;
    }

    if (defined($page->{'controller'})) {
	$page->gui->message(name => Controller::EncodeState, to => $page->{'controller'}, parms => { monitor => 1 });
    }

    $page->aspect;
    return 1;
}

sub deattach {
    my $page = shift;

    $page->SUPER::deattach(@_);

    if (defined($page->{'controller'})) {
	$page->gui->message(name => Controller::EncodeState, to => $page->{'controller'}, parms => { demonitor => 1 });
    }
    return 1;
}

sub configure {
    my $page = shift;
    my %parms = @_;

    $page->{'controller'} = $parms{'controller'} || die;
    $page->gui->monitor($page->{'controller'});
    if (!$page->hidden && ($page->{'controller'}->currentState eq Thing::Handle::Alive)) {
	$page->gui->message(name => Controller::EncodeState, to => $page->{'controller'}, parms => { monitor => 1 });
    }

    return 1;
}

sub monitorChange {
    my $page = shift;
    my $handle = shift;

    if (defined($page->{'controller'}) && ($handle == $page->{'controller'}) && !$page->hidden && ($handle->currentState eq Thing::Handle::Alive)) {
	$page->gui->message(name => Controller::EncodeState, to => $page->{'controller'}, parms => { monitor => 1 });
    }

    return 1;
}

sub aspect {
    my $page = shift;
    return 1;
}

# encodeStatus

sub encodeStatus {
    my $page = shift;
    my $parms = shift;

    my @rates = ();
    my $secondsToEncode = 0;
    my $pos = 0;
    my $list = $page->{'trackCList'};
    $list->freeze;
    $list->clear;
    foreach my $ti (@{ $parms->{'tracks'} }) {
	my $track = $ti->{'track'};
	my $album = $track->album;
	my $artist = $album->artist;
	my $rate = $ti->{'rate'};
	$list->append(sprintf('%d. [%d:%02d]', $pos + 1, int($track->duration / 60), int($track->duration % 60)),
		      $track->name, $artist->name, $album->name, $rate);
	$list->set_selectable($pos, 0);
	$ti->{'pos'} = $pos + 1;
	$pos++;
	$secondsToEncode += $track->duration;
    }
    $list->thaw;

    $pos = 0;
    $list = $page->{'encoderCList'};
    $list->freeze;
    $list->clear;
    foreach my $encoder (@{ $parms->{'encoders'} }) {
	my $name = ucfirst $encoder->{'handle'}->key;
	my $desc; 
	if ($encoder->{'state'} eq Controller::Encoding) {
	    # Match the track with this encoder
	    my @tracks = grep { defined($_->{'handle'}) && Thing::Handle::cmpNameHost($_->{'handle'}, $encoder->{'handle'}) }
                              @{ $parms->{'tracks'} };
	    if ($#tracks == 0) {
		$desc = 'Encoding track '.$tracks[0]->{'pos'};
		# Work out how fast it is going
		my $secondsEncoded = ($tracks[0]->{'track'}->duration * $tracks[0]->{'percentageComplete'}) / 100;
		my $timeToEncode = time - $tracks[0]->{'start'};
		if ($secondsEncoded && ($timeToEncode > 0)) {
		    my $rate = $secondsEncoded / $timeToEncode;
		    push @rates, $rate;
		}
		$secondsToEncode += ($tracks[0]->{'track'}->duration - $secondsEncoded);
		$desc .= ', '.int($tracks[0]->{'percentageComplete'}).'%, since '.
		    POSIX::strftime('%c', localtime($tracks[0]->{'start'}));
	    }
	    else {
		Trace::error('Cannot find encoding track');
		$desc = '';
	    }
	}
	elsif ($encoder->{'state'} eq Controller::Dead) {
	    $desc .= 'Encoder is dead';
	}
	elsif ($encoder->{'state'} eq Controller::Halted) {
	    $desc .= 'Encoder is halted';
	}
	elsif ($encoder->{'state'} eq Controller::Idle) {
	    $desc .= $encoder->{'reason'};
	}
	$list->append($name, $desc);
	$list->set_selectable($pos, 0);
	$pos++;
    }
    $list->thaw;

    my $button = $page->{'haltButton'};
    my $label = $page->{'haltLabel'};
    if ($parms->{'state'} eq Controller::Running) {
	$label->set_text('Halt');
    }
    else {
	$label->set_text('Resume');
    }

    if ($#rates >= 0) {
	my $rate;
	map { $rate += $_; } @rates;
	my $seconds = int($secondsToEncode / $rate);
	#my $est;
	#$est .= int($seconds / 86400).'d ';
	#$est .= (int($seconds / 3600) % 24).'h ';
	#$est .= (int($seconds / 60) % 60).'m ';
	#$est .= ($seconds % 60).'s.';
	$page->{'infoLabel'}->set_text(POSIX::strftime('Should finish by %c.', localtime(time + $seconds)));
	$page->{'infoLabel'}->show;
    }
    else {
	$page->{'infoLabel'}->hide;
    }

    $page->{'state'} = $parms->{'state'};
}

# haltOrResume

sub haltOrResume {
    my $page = shift;
    my $widget = shift;

    if ($page->{'state'} eq Controller::Running) {
	$page->gui->message(name => Controller::EncodeHalt, to => $page->{'controller'});
    }
    else {
	$page->gui->message(name => Controller::EncodeResume, to => $page->{'controller'});
    }
    return 1;
}

1;
