# Copyright (C) 2000 Gavin Jefferies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Revision: 1.3 $

package Gui::Window;

# Windows that form the Gui.

use GtkEx::BusyIcon;
use strict;
use Trace;

sub new {
    my $proto = shift;
    my $type = ref($proto) || $proto;
    my %parms = @_;
    my $window = {};
    my $data = $window->{'_window'} = {};

    bless $window, $type;

    $data->{'page'} = $parms{'page'} || die;
    $data->{'level'} = $parms{'level'} || die;
    $data->{'id'} = $parms{'id'} || die;
    my $gui = $data->{'gui'} = $parms{'gui'} || die;

    my $win = Gtk::Window->new($data->{'level'});
    $data->{'window'} = $win;
    $data->{'destroySid'} = $win->signal_connect('destroy', sub { $window->destroy(@_); });
    $data->{'deleteSid'} = $win->signal_connect('delete_event', sub { $window->destroy(@_); });
    $win->border_width(0);
    $data->{'width'} = $window->page->default_width;
    $data->{'height'} = $window->page->default_height;
    $win->set_default_size($data->{'width'}, $data->{'height'});
    $win->set_title('emptytree-seedy - '.$window->page->title);
    $win->show();

    # Debug and window configure

    $win->signal_connect('event', sub { $window->event(@_); });

    # Accelerators

    $data->{'accelGroup'} = my $accelGroup = Gtk::AccelGroup->new;
    $win->add_accel_group($accelGroup);

    $data->{'container'} = my $vbox = Gtk::VBox->new(0, 0);
    $vbox->border_width(0);
    $vbox->show();
    $win->add($vbox);

    my $topbar = $gui->topbarCreate(window => $window);
    $data->{'busy'} = $topbar->{'busy'};
    $vbox->pack_start($topbar, 0, 0, 0);

    my $hsep = Gtk::HSeparator->new;
    $hsep->show;
    $vbox->pack_start($hsep, 0, 0, 0);

    # Page

    $vbox->pack_start($window->page->widget, 1, 1, 0);
    $window->page->attach(window => $window);

    return $window;
}

sub page          { $_[0]->{'_window'}->{'page'} }
sub window        { $_[0]->{'_window'}->{'window'} }
sub id            { $_[0]->{'_window'}->{'id'} }
sub gui           { $_[0]->{'_window'}->{'gui'} }
sub busy          { $_[0]->{'_window'}->{'busy'}; }
sub accelGroup    { $_[0]->{'_window'}->{'accelGroup'}; }
sub width         { $_[0]->{'_window'}->{'width'}; }
sub height        { $_[0]->{'_window'}->{'height'}; }

# set_title(newtitle)

sub set_title {
    my $window = shift;
    $window->window->set_title('emptytree-seedy - '.shift);
}

# destroy - closed, deleted.

sub destroy {
    my $window = shift;
    my $data = $window->{'_window'};

    $window->page->deattach;
    $data->{'container'}->remove($window->page->widget);

    $window->gui->windowDestroyed(window => $window); 

    $window->window->signal_handler_block($data->{'deleteSid'});
    $window->window->signal_handler_block($data->{'destroySid'});
    $window->window->destroy;

    return 1;
}

# event - Debug/catch signals/events

sub event {
    my $window = shift;
    my $data = $window->{'_window'};
    my $widget = shift;
    my $event = shift;

    if ($event->{'type'} eq 'configure') {
	if (($data->{'width'} != $event->{'width'}) || ($data->{'height'} != $event->{'height'})) {
	    $data->{'width'} = $event->{'width'};
	    $data->{'height'} = $event->{'height'};
	    $window->page->aspect;
	}
    }

    if ($ENV{GUI_EVENT_DEBUG}) {
	Trace::debug(ref($widget).'::'.$event->{'type'}) if Trace::check(Trace::DEBUG3);
	if ($ENV{GUI_EVENT_DEBUG} eq 'v') {
	    require 'dumpvar.pl';
	    main::dumpValue($event);
	    $| = 1; 
	}
    }

    return 0;
}

1;
